
#pragma once

#include <vector>
#include "ArchiveManager.h"
#include "7zip/UI/Common/LoadCodecs.h"
#include "7zip/Archive/IArchive.h"

typedef struct {
	UInt32  Index;
	UInt64  Size;
	UString Name;
	UString Path;
} FileInfo;

typedef std::vector<FileInfo*> FileList;
typedef const FileList ConstFileList;

class CArchiveExtractor
{
private:
	FileList fileList_;
	CMyComPtr<IInArchive> archive_;
	int archiveIndex_;
	CMyComPtr<CInFileStream> file_;
	bool closeFileHandle_;

public:
	CArchiveExtractor();
	~CArchiveExtractor();

	int GetArchiveIndex() { return archiveIndex_; }

	bool Open(CCodecs *codecs, HANDLE hFile);
	bool OpenAs(CCodecs *codecs, HANDLE hFile, int index);
	bool Open(CCodecs *codecs, const WCHAR *file);
	bool OpenAs(CCodecs *codecs, const WCHAR *file, int index);
private:

	bool Open(CCodecs *codecs, CMyComPtr<CInFileStream> file);
	bool OpenAs(CCodecs *codecs, CMyComPtr<CInFileStream> file, int index);

public:
	bool ExtractToStream(UInt32 index, ISequentialOutStream *stream, ArchiveExtractCallback func, LPVOID param);
	bool ExtractToFile(UInt32 index, const WCHAR *file, ArchiveExtractCallback func, LPVOID param);
	bool ExtractToMemory(UInt32 index, BYTE *buff, UInt32 buffSize, ArchiveExtractCallback func, LPVOID param);
	bool ExtractToHandle(UInt32 index, HANDLE handle, ArchiveExtractCallback func, LPVOID param);

	const FileList & GetFileList() const { return fileList_; }
private:
	bool makeFileList();

};

